@@ -1,30 +1,58 @@
 package Log::Declare;
+# ABSTRACT: A high performance Perl logging module.
 
+use 5.10.0; # for //
 use strict;
+use warnings;
+
 use Devel::Declare::Lexer;
 use Devel::Declare::Lexer::Token::Raw;
 use POSIX qw(strftime);
 use Data::Dumper; # for d: statements
 
-our $VERSION = '0.03';
+our $VERSION = '0.10';
+
+my %LEVEL = (
+    ALL     => -1,
+    TRACE   =>  1,
+    DEBUG   =>  2,
+    INFO    =>  3,
+    WARN    =>  4,
+    ERROR   =>  5,
+    AUDIT   =>  6,
+    OFF     =>  7,
+    DISABLE =>  7,
+);
+
+# XXX be careful about removing/renaming this: it's required by MojoX::Log::Declare
+our @level_priority = qw(audit error warn info debug trace);
+
+my ($LEVEL, $LEVEL_NAME);
+__PACKAGE__->startup_level($ENV{'LOG_DECLARE_STARTUP_LEVEL'} || 'ERROR'); # sets $LEVEL and $LEVEL_NAME
 
-my $startup_level = uc ($ENV{'LOG_DECLARE_STARTUP_LEVEL'} || 'ERROR');
 my $log_statement = "Log::Declare->log('%s', [%s], %s)%s";
 
 unless($ENV{LOG_DECLARE_NO_STARTUP_NOTICE}) {
-    Log::Declare->log('INFO', ['LOGGER'], 'Got logger startup level of ' . $startup_level);
+    Log::Declare->log('INFO', ['LOGGER'], "Got logger startup level of $LEVEL_NAME");
 }
 
-# These provide return values for injected keywords - a 0 here means the level
-# is completely disabled and won't be received by the log writer
-our @level_priority = ( 'info', 'error', 'warn', 'debug', 'trace' );
-our %levels = (
-    info  => sub { 1 },
-    error => sub { 1 },
-    warn  => sub { 1 },
-    debug => sub { 1 },
-    trace => sub { 1 },
-);
+# this provides a way to globally override the behaviour of the injected keywords.
+# if replaced by e.g. a sub which returns 0, the level will be completely disabled and
+# the log writer won't be called. The original implementations can be restored at any
+# time by deleting the hooks.
+# XXX be careful about removing/renaming this: it's required for namespace hooks (see
+# the NAMESPACES section in the POD).
+our %levels;
+
+# define the exported trace, debug &c. subs. These delegate to the hooked implementations
+# in %levels (if defined); otherwise they return true/false if the level is enabled/disabled
+my %EXPORT;
+for my $name (@level_priority) {
+    my $hook;
+    my $level = $LEVEL{uc $name};
+    # goto &sub: make sure caller() works as expected in the hooked sub
+    $EXPORT{$name} = sub { ($hook = $levels{$name}) ? goto &$hook : $level >= $LEVEL };
+}
 
 BEGIN {
     my $callback = sub {
@@ -118,7 +146,7 @@ BEGIN {
                 if(ref($token) =~ /Devel::Declare::Lexer::Token::Comma/) {
                     push @categories, (uc "\"$buf\"") if $buf;
                     $buf = '';
-                    next; 
+                    next;
                 }
                 next if $buf eq '' && ref($token) =~ /Devel::Declare::Lexer::Token::Whitespace/;
                 $buf .= $token->{value};
@@ -182,8 +210,9 @@ BEGIN {
     };
 
     # Setup callbacks for each of the keywords
-    Devel::Declare::Lexer::lexed(info => $callback);
-    Devel::Declare::Lexer::lexed(warn => $callback);
+    Devel::Declare::Lexer::lexed(audit => $callback);
+    Devel::Declare::Lexer::lexed(info  => $callback);
+    Devel::Declare::Lexer::lexed(warn  => $callback);
     Devel::Declare::Lexer::lexed(error => $callback);
     Devel::Declare::Lexer::lexed(debug => $callback);
     Devel::Declare::Lexer::lexed(trace => $callback);
@@ -191,12 +220,19 @@ BEGIN {
 
 # -----------------------------------------------------------------------------
 
+# set the global log level
+# FIXME this should be called level
 sub startup_level {
-    my ($self, $level) = @_;
+    my $self = shift;
 
-    return $startup_level unless $level;
-    $startup_level = $level;
-    return $startup_level;
+    if (@_) {
+        my $level = shift // '';
+        $LEVEL_NAME = uc $level;
+        # ALL: be forgiving if the name is invalid/mistyped (see below)
+        $LEVEL = $LEVEL{$LEVEL_NAME} // $LEVEL{ALL};
+    } else {
+        return $LEVEL_NAME;
+    }
 }
 
 # -----------------------------------------------------------------------------
@@ -212,28 +248,28 @@ sub log_statement {
 # -----------------------------------------------------------------------------
 
 sub log {
-    my ($self, $level, $categories, $message) = @_;
+    my ($self, $level_name, $categories, $message) = @_;
+
+    $level_name = uc($level_name // '');
+
+    # be forgiving if the log level is mistyped/invalid: it's going
+    # to be easier to remove an unwanted log message than to track
+    # down a bug that isn't being logged because of a typo
+    my $level = $LEVEL{$level_name} // $LEVEL;
+
+    return unless $level >= $LEVEL;
 
     if($categories) {
         $categories = scalar @$categories > 0 ? (join ', ', @$categories) : '';
         $categories = " [$categories]";
     }
 
-    my $allowed = 0;
-    for my $lvl (@Log::Declare::level_priority) {
-        $allowed = 1 if $lvl eq $level;
-        last if uc $lvl eq uc $startup_level;
-    }
-
-    return if !$allowed;
-
     my $ts = strftime $ENV{'LOG_DECLARE_DATE_FORMAT'} // "%a %b %e %H:%M:%S %Y",
                       ($ENV{'LOG_DECLARE_USE_LOCALTIME'} ? localtime : gmtime);
 
     $message .= "\n" if substr($message,-1) ne "\n";
 
-    $level = uc $level;
-    return CORE::print STDERR "$$ [$ts] [$level]$categories $message";
+    return CORE::print STDERR "$$ [$ts] [$level_name]$categories $message";
 }
 
 # -----------------------------------------------------------------------------
@@ -277,12 +313,12 @@ sub do_import {
     my %t = map { $_ => 1 } @tags;
     return if $t{':nosyntax'};
 
-    # Inject each of the keywords into callers namespace
-    Devel::Declare::Lexer::import_for($caller, { 'error' => sub { goto $Log::Declare::levels{'error' } } } ) if !$t{':noerror'};
-    Devel::Declare::Lexer::import_for($caller, { 'trace' => sub { goto $Log::Declare::levels{'trace' } } } ) if !$t{':notrace'};
-    Devel::Declare::Lexer::import_for($caller, { 'debug' => sub { goto $Log::Declare::levels{'debug' } } } ) if !$t{':nodebug'};
-    Devel::Declare::Lexer::import_for($caller, { 'warn'  => sub { goto $Log::Declare::levels{'warn'  } } } ) if !$t{':nowarn' };
-    Devel::Declare::Lexer::import_for($caller, { 'info'  => sub { goto $Log::Declare::levels{'info'  } } } ) if !$t{':noinfo' };
+    # Inject each of the keywords into the caller's namespace
+    for my $name (@level_priority) {
+        Devel::Declare::Lexer::import_for($caller, {
+            $name => $EXPORT{$name}
+        }) if !$t{":no$name"};
+    }
 }
 
 # -----------------------------------------------------------------------------
@@ -305,7 +341,7 @@ the logging is disabled:
 
     $self->log(Dumper $myobject);
 
-but with Log::Declare we incur almost no performance penalty if 'info' level is 
+but with Log::Declare we incur almost no performance penalty if 'info' level is
 disabled, since the following log statement:
 
     info Dumper $myobject [mycategory];
@@ -342,7 +378,7 @@ which means if 'info' returns 0, nothing else gets evaluated.
 
 =head1 NAMESPACES
 
-If you're using a namespace aware logger, Log::Declare can use your loggers
+If you're using a namespace-aware logger, Log::Declare can use your logger's
 namespacing to determine log levels. For example:
 
     $Log::Declare::levels{'debug'} = sub {
